import { ModelPersistenceEventCallback } from '../Types';
/**
 * Attribute index configuration
 */
export interface IModelAttributeIndex {
    name: string;
    unique?: boolean;
    isSecondaryKey?: boolean;
    secondaryKey?: string;
}
/**
 * Model attribute options
 */
export interface IModelAttributeOptions {
    name?: string;
    type?: any;
    typeName?: string;
    isArray?: boolean;
    primaryKey?: boolean;
    secondaryKey?: boolean;
    index?: IModelAttributeIndex;
    transient?: boolean;
}
/**
 * Options provided to model
 * decorator annotation
 */
export interface IModelOptions {
    clazzName?: string;
    clazz?: any;
    onlyMapDefinedAttributes?: boolean;
    tableName?: string;
    attrs?: IModelAttributeOptions[];
    transientAttrs?: string[];
    onPersistenceEvent?: ModelPersistenceEventCallback<any>;
}
/**
 * Model key shape
 */
export interface IModelKey {
}
/**
 * Model key value shape
 */
export interface IKeyValue {
}
export declare type TKeyValue = IKeyValue | string | number;
/**
 * Decorate a specified class, making it a
 * PersistableModel
 *
 * Set process.env.DYNO_SKIP to true in order to skip
 * decorations - useful in dual purpose classes,
 * in webpack use DefinePlugin
 *
 * @param opts
 */
export declare function Model(opts?: IModelOptions): (constructor: Function) => void;
/**
 * Default value resolver
 */
export declare type DefaultValueFn = (o: any) => any;
/**
 * Default value decoration
 */
export declare function DefaultValue(defaultValueFn: DefaultValueFn): (target: any, propertyKey: string) => void;
/**
 * Decorator model attribute
 *
 * @param opts
 * @returns {function(any, string): undefined}
 * @constructor
 */
export declare function Attribute(opts?: IModelAttributeOptions): (target: any, propertyKey: string) => void;
